/*
*****************************************************************************
  *		RBus2.h - library for R-Bus protocoll on Serial_port_2 
  *		Copyright (c) 12/2016 Mitin Alexey  All right reserved.
  *
  *		Version 1.1      17.12.16
*****************************************************************************
*/

// include this library's description file
#include "RBus2.h"
#include <avr/interrupt.h>

// Constructor /////////////////////////////////////////////////////////////////
// Function that handles the creation and setup of instances

RBus2Class::RBus2Class() {
// initialize this instance's variables 
// XNetclear0();	//alte Nachricht lschen
// XNetclear1();	//alte Nachricht lschen
}

//******************************************Serial*******************************************
void RBus2Class::setup(uint8_t  XControl)  //Initialisierung Serial  //edit 17.12.16
{
	MAX485_CONTROL_2 = XControl;
	// LISTEN_MODE 
	pinMode(MAX485_CONTROL_2, OUTPUT);
	digitalWrite (MAX485_CONTROL_2, LOW);

	//Set up on 62500 Baud
	cli();  //disable interrupts while initializing the USART

	 UBRR2H = 0;
	 UBRR2L = 0x0F;
	 UCSR2A = 0;
	 UCSR2B = (1<<RXEN2) | (1<<TXEN2) | (1<<RXCIE2) | (1<<UCSZ22);
	 UCSR2C = (1<<UCSZ21) | (1<<UCSZ20);

	sei(); // Enable the Global Interrupt Enable flag so that interrupts can be processed 
	 /*
	 *  Enable reception (RXEN = 1).
	 *  Enable transmission (TXEN0 = 1). 
	 *	Enable Receive Interrupt (RXCIE = 1).
	 *  Set 8-bit character mode (UCSZ00, UCSZ01, and UCSZ02 together control this, 
	 *  But UCSZ00, UCSZ01 are in Register UCSR0C).
	 */
}

// Public Methods //////////////////////////////////////////////////////////////
// Functions available in Wiring sketches, this library, and other libraries

//*******************************************************************************************

//--------------------------------------------------------------------------------------------
// send along a bunch of bytes to the Command Station
void RBus2Class::XNetsend(unsigned int *dataString, byte byteCount) {     //edit 17.12.16

	digitalWrite(MAX485_CONTROL_2, HIGH); 	//SEND_MODE
	for (byte i = 0; i < byteCount; i++) {
		XNetTransmit9(*dataString);   	//Data String
		dataString++;
	}
	WAIT_FOR_XMIT_COMPLETE_2;
	digitalWrite(MAX485_CONTROL_2, LOW);	//LISTEN_MODE
	// XSendCount = micros();		//Reset Sendercount for next Call Byte  //???
}

//--------------------------------------------------------------------------------------------
// send byte to the Command Station
void RBus2Class::XNetsend1(unsigned int databyte) {    //edit 17.12.16
   delayMicroseconds(2);
   digitalWrite (MAX485_CONTROL_2, HIGH);
   delayMicroseconds(2);
   XNetTransmit9(databyte);	//Data Byte
   WAIT_FOR_XMIT_COMPLETE_2;
   digitalWrite (MAX485_CONTROL_2, LOW);
   // XSendCount = micros();		//Reset Sendercount for next Call Byte  //???
}

//--------------------------------------------------------------------------------------------
// calculate the XOR
void RBus2Class::getXOR (unsigned int *data, byte length) {
  byte XOR = 0x00;
	data++;		//without CallByte
	for (byte i = 0; i < (length-2); i++) { 
        XOR = XOR ^ *data;
		data++;
    }
  XNetsend1(XOR);
  XOR = 0x00;		//Reset XOR
}

//--------------------------------------------------------------------------------------------
// Transmit9
void RBus2Class::XNetTransmit9(unsigned int data8)   //edit 17.12.16
{
		/* wait for empty transmit buffer */
		while (!(UCSR2A & (1 << UDRE2))) {}
		/* put the data into buffer, and send */
		UCSR2B &= ~(1 << TXB82);
		if (data8 & 0x100) 
			UCSR2B |= (1 << TXB82);
		UDR2 = data8;
}


